﻿//1. Module Name
module asteroids;

//2. Arc Imports and Game Imports 
import
	arc.input,
	arc.window,
	arc.time,
	arc.font,
	arc.gui.gui,
	arc.physics.physics,
	arc.physics.physics,
	arc.scenegraph.all;

import 
	ship,
	asteroid,
	laserbolt,
	spacearea,
	starfield,
	scoreboard, 
	gamegui;
	
import tango.io.Stdout;
import tango.util.log.Configurator;
		
//3. The starfield will draw the background stars, level will keep track of level user is on
int level = 1; 

//4. Main
void main()
{
	//6. Open audio
	try
		arc.sound.open();
	catch(Exception e)
		Stdout("There was an error initializing the sound, continuing anyway...").newline;

	//5. Open window with title, width, height, and fullscreen-flag
	arc.window.open("Asteroids", 800, 600, false);
	
	//7. initialize input handling, output logging and font handling
	arc.input.open();
	arc.font.open();
	arc.physics.physics.open();
	
	//8. load layout data
	gamegui.load();    
	
	//9. reset game to start it up
	setupLevel(level); 
        
	//11. loop until the user wants to quit
	while (!(arc.input.keyDown(arc.input.ARC_QUIT)))
	{
		//12. this works in conjunction with arc.time.limitFPS below
		// basically, it grabs the time at the beginning of this frame
		// and updates the fps-count
		arc.time.process();
		
		//13. updates data about keyboard, mouse and joysticks
		// you can either pull the data via arc.input.keyPressed and similar
		// or subscribe to changes using the signals in arc.input.signals
		arc.input.process();

		//14. clear the current screen
		arc.window.clear();
        
		//15. run game processing if the game is not paused 
		if (!gamegui.gamePaused)
		{
			//16. process players ship
			playerShip.process();
			
			//17. process sound effects 
			arc.sound.process();
			
			//18. Goes through the scenegraph, calling the advance method on every
			// node that's derived from IAdvancable.
			// The physics engine will process the collisions and movements of the ship and the asteriods
			// and animations will update their frames if the delay is over.
			arc.scenegraph.advancable.advanceScenegraph(arc.time.elapsedMilliseconds());
		}

		//19. Goes through the scenegraph and calls the draw method on every node
		// that is derived from IDrawable. Actually, it does a bit more - it respects
		// the draw order layed out by DrawOrder nodes, but basically it just calls draw
		// everywhere.
		arc.scenegraph.drawable.drawScenegraph();
 
		//20. If no asteroids are left, advance to the next level 
		if (Asteroid.count == 0)
		{
			setupLevel(++level); 
		}			
		
		//21. takes a screenshot if the s key is hit
		if (arc.input.keyPressed('s'))
			arc.window.screenshot("screen"); 
		
/*		foreach(c; &rootNode.traverseDown)
			Stdout(c.classinfo.name).newline;
		Stdout(Asteroid.count).newline;
*/		
		//22. process and draw GUI
		gamegui.process();
		gui.process();
		gui.draw(); 
        
		//23. Swaps window buffers, showing what we just drew to the user
		arc.window.swap();
		
		//24. Limits the frames per second to a given value; can be used to simulate slow computers
		// and to make sure the program doesn't run too fast on fast machines
		arc.time.limitFPS(40);
	}	
    
	//25. this will be called even on exe crash or assert, so
	// it is a good place to clean up
	scope(exit)
	{
		// close what we opened above
		arc.input.close();
		arc.font.close();
		arc.sound.close();
		arc.window.close();
	}
}

//26. reset game 
void setupLevel(uint levelNum, bool resetScore=false)
{
	//7. Create our ship, asteroids, stars, and score
	int oldScore=0;
	
	if (scoreBoard !is null)
	{
		oldScore = scoreBoard.getScore;
	}
	
	// remove everything from the root node
	rootNode.disconnectChildren();

	// create new classes, set things up
	stars = new Starfield; 
	area = new SpaceArea;
	playerShip = new Ship;
	
	scoreBoard = new Scoreboard; 

	if (!resetScore)
	{
		scoreBoard.addScore(oldScore); 
	}
		
	area.setupAsteroids(levelNum);
}
