//1. module name
module gamegui; 

//2. Imports
import arc.gui.gui;
import arc.math.point;
import arc.math.size; 
import asteroids;
import arc.input; 

//3. Bools, GUI, and GUI code handler 
bool gamePaused = false;
bool showGUI = false; 
bool showOptions = false; 
bool showAudio = false;
bool showVideo = false;

GUI gui; 
HandleGUI handle;

//4. The HandleGUI class
class HandleGUI
{
	// main menu code //////////
	//5. exit the game
	void exit()
	{
		// exit application
		arc.input.quit(); 
	}

	//6. return to current game
	void returnToGame()
	{
		showGUI = false; 
		gamePaused = false;
	}

	//7. start a new game 
	void newGame()
	{
		asteroids.setupLevel(asteroids.level=1, true); 
		showGUI = false;
		gamePaused = false;
	}

	//8. show the options
	void options()
	{
		showGUI = false;
		showOptions = true; 
	}

	// options code ///////////
	//9. Show video options
	void video()
	{
		showOptions = false;
		showVideo = true;
	}

	//10. Show audio options 
	void audio()
	{
		showOptions = false;
		showAudio = true;
	}

	//11. Go back to the previous screen 
	void back()
	{
		showOptions = false;
		showGUI = true;
	}

	// audio code //////////
	//12. Turn sound on 
	void on()
	{
		// turn sound on
		arc.sound.on();
		showAudio = false;
		gamePaused = false;
	}

	//13. Turn sound off 
	void off()
	{
		// turn sound off 
		arc.sound.off();
		showAudio = false;
		gamePaused = false;
	}

	//14. Go to the previous menu 
	void backAud()
	{
		showAudio = false;
		showOptions = true; 
	}

	// video code //////////
	//15. Resize window to 600x480 resolution
	void six()
	{
		arc.window.resize(600, 480);
		showVideo = false;
		gamePaused = false;
	}

	//16. Resize window to 800x600 resolution
	void eight()
	{
		arc.window.resize(800, 600);
		showVideo = false;
		gamePaused = false;
	}

	//17. Resize window to 1024x768 resolution
	void thousand()
	{
		arc.window.resize(1024, 768);
		showVideo = false;
		gamePaused = false;
	}

	//18. Toggle full screen 
	void fullscreen()
	{
		arc.window.toggleFullScreen();
		showVideo = false;
		gamePaused = false;
	}

	//19. Go back to the previous screen 
	void backVid()
	{
		showVideo = false;
		showOptions = true;
	}
}


//20. load game gui
void load()
{
	handle = new HandleGUI; 
	gui = new GUI("astbin/gui/gui.xml");

	gui.getLayout("main").setPosition(Size.toPoint(arc.window.getSize/2 - Point(100,100))); 
	gui.getLayout("options").setPosition(Size.toPoint(arc.window.getSize/2 - Point(100,75)));
	gui.getLayout("video").setPosition(Size.toPoint(arc.window.getSize/2 - Point(100,125)));
	gui.getLayout("audio").setPosition(Size.toPoint(arc.window.getSize/2 - Point(100,75)));


	// hook main menu code up 
	gui.getLayout("main").getWidget("exit").clicked.attach(&handle.exit);
	gui.getLayout("main").getWidget("return").clicked.attach(&handle.returnToGame);
	gui.getLayout("main").getWidget("newgame").clicked.attach(&handle.newGame);
	gui.getLayout("main").getWidget("options").clicked.attach(&handle.options); 
	// hook up options code
	gui.getLayout("options").getWidget("video").clicked.attach(&handle.video);
	gui.getLayout("options").getWidget("audio").clicked.attach(&handle.audio);
	gui.getLayout("options").getWidget("back").clicked.attach(&handle.back);
	// hook up audio code
	gui.getLayout("audio").getWidget("on").clicked.attach(&handle.on);
	gui.getLayout("audio").getWidget("off").clicked.attach(&handle.off);
	gui.getLayout("audio").getWidget("back").clicked.attach(&handle.backAud);
	// hook up video code
	gui.getLayout("video").getWidget("600").clicked.attach(&handle.six);
	gui.getLayout("video").getWidget("800").clicked.attach(&handle.eight);
	gui.getLayout("video").getWidget("1024").clicked.attach(&handle.thousand);
	gui.getLayout("video").getWidget("fullscreen").clicked.attach(&handle.fullscreen);
	gui.getLayout("video").getWidget("back").clicked.attach(&handle.backVid);
    
}

///21. process game gui 
void process()
{
	if (arc.input.keyPressed(ARC_ESCAPE) && !gamegui.showOptions && !gamegui.showAudio && !gamegui.showVideo)
	{
		gamegui.showGUI = !gamegui.showGUI; 
		gamePaused = !gamePaused;
	}

	if (showGUI)
	{
		gui.getLayout("main").setHide(false);
	}
	else
	{
		gui.getLayout("main").setHide(true);
	}

	if (showOptions)
	{
		if (arc.input.keyPressed(ARC_ESCAPE))
		{
			showOptions = false;
			showGUI = true;
		}
		
		gui.getLayout("options").setHide(false);
	}
	else
		gui.getLayout("options").setHide(true);

	if (showAudio)
	{
		if (arc.input.keyPressed(ARC_ESCAPE))
		{
			showAudio = false;
			showOptions = true;
		}
		
		gui.getLayout("audio").setHide(false);
	}
	else
		gui.getLayout("audio").setHide(true);

	if (showVideo)
	{
		if (arc.input.keyPressed(ARC_ESCAPE))
		{
			showVideo = false;
			showOptions = true;
		}
		
		gui.getLayout("video").setHide(false);
	}
	else
		gui.getLayout("video").setHide(true);
}