//1. Module name
module laserbolt;

//2. Imports 
import
	arc.scenegraph.all,
	arc.physics.shapes.box,
	arc.sound,
	arc.texture,
	arc.types,
	arc.window;
	
import spacearea;

//3. A laser bolt, fired from the player ship.
class LaserBolt : Node
{
public:
	
	//5. creates a new laserbolt at a given position, rotation
	// and adds it to the spacearea node.
	static LaserBolt addLaser(inout Point translation, Radians rotation)
	{
		LaserBolt bolt = new LaserBolt(translation, rotation);
		return bolt;
	}
	
private:
	//6. this is serves as a callback to the physics collision handler
	// it is registered in the constructor and will be called whenever
	// there's a new collision between this object and another body
	// 
	// if a laserbolt hits something, remove it
	void onCollide(Body a, Body b, arcfl msDt)
	{
		setParent(null);
		physics.sigSustainedPostCollide.detach(&onCollide);
	}
	
	//7. new laser at position and angle
	this(inout Point translation, Radians rotation)
	{
		// LaserBolt derives from Sprite and can call Sprite's
		// constructor with a Transform and a 'content' node.
		// Since we want the laser bolt to collide with objects, we
		// use one of the physics nodes as a transform:
		
		physics = new Box(this, image.frame.size, 10);
		physics.sigSustainedPostCollide.attach(&onCollide);
		physics.translation = translation;
		physics.rotation = rotation;
		physics.velocity = Point.fromPolar(0.5, rotation - PI/2);
		
		// the 'content' will just be the previously constructed
		// beamFrame - it just draws the beam
		
		image.makeNode(physics);
		
		super(spacearea.area);
	}
		
private:
	//8. Laser bolt vars 
	Body physics;
	static Image image;

	//4. loads the graphics for the bolt
	static void loadResources()
	{
		// Frame is derived from MultiParentNode, so we will only need
		// to have one instance, which will serve to display the bolt
		// for all instances of LaserBolt
		LaserBolt.image = new Image(Texture("astbin/ship/laser.png"));
	}	
	mixin onWindowOpen!(loadResources);
}