﻿//1. Declare module name
module ship; 

//2. The ship will need to make use of all these imports, plus we define playerShip class global 
import
	arc.window,
	arc.texture,
	arc.sound,
	arc.input,
	arc.scenegraph.all,
	arc.physics.shapes.circle,
	arc.types,
	arc.particle;

import
	tango.core.Signal;

import asteroid;
import laserbolt;

Ship playerShip;

class ShipExplosion : Node, IDrawable, IAdvancable
{
	this(Node parent_)
	{
		super(parent_);
		
		scope col = makeTriangleRandom!(uniform1D)(Color.Red, Color.Yellow, Color(1., 0.5, 0.1));
		
		scope debrisvel = makeBallRandom!(risingLinear1D)(Point(0,0), 150.f);
		Particle debrisParticle()
		{
			Particle p;
			p.size = Size(6, 6);
			col.set(p.color);
			debrisvel.set(p.velocity);
			return p;
		}		
		arc.particle.spawn(debris, 20u, &debrisParticle);
		
		scope glowpos = makeBallRandom!(fallingLinear1D)(Point(0,0), 32f);
		scope glowvel = makeBallRandom!(fallingLinear1D)(Point(0,0), 16f);
		Particle glowParticle()
		{
			Particle p;
			p.size = Size(16,16);
			glowpos.set(p.position);
			glowvel.set(p.velocity);
			col.set(p.color);
			return p;
		}		
		arc.particle.spawn(glow, 400u, &glowParticle);
		
		arcfl outer1D() { return 1.; }		
		scope ringvel = makeBallRandom!(outer1D)(Point(0,0), 200.f);
		Particle ringParticle()
		{
			Particle p;
			p.size = Size(8, 8);
			ringvel.set(p.velocity);
			p.color = Color(1., 1., 1., 0.5);
			return p;			
		}
		arc.particle.spawn(ring, 1000u, &ringParticle);
	}
	
	void draw()
	{
		Texture drawtex = Texture("astbin/particle16.png");
	
		arc.particle.draw(glow, drawtex);
		arc.particle.draw(debris, drawtex);
		arc.particle.draw(ring, drawtex);
	}
	
	void advance(arcfl msDt)
	{
		arcfl sDt = msDt / 1000.;
		
		arc.particle.age(debris, sDt);
		arc.particle.killOldAge(debris, 2.0f);
		arc.particle.fade(debris, 2.0f);
		arc.particle.move(debris, sDt);
		
		arc.particle.age(glow, sDt);
		arc.particle.killOldAge(glow, 4.0f);
		arc.particle.fade(glow, 4.0f);
		arc.particle.move(glow, sDt);		
		
		arc.particle.age(ring, sDt);
		arc.particle.killOldAge(ring, 2.0f);
		arc.particle.fade(ring, 2.0f);
		arc.particle.move(ring, sDt);
		
		if(arc.particle.aliveCount(debris) == 0 && arc.particle.aliveCount(glow) == 0)
		{
			setParent(null);	
			sigEnd();
		}
	}
	
	Signal!() sigEnd;
	
private:
	struct Particle
	{
		Point position = Point(0,0);
		Point velocity;
		Size size = Size(32,32);
		Color color = Color.White;
		
		bool alive = true;
		arcfl age = 0.;
	}
	
	Particle[] debris;
	Particle[] glow;
	Particle[] ring;
}

//3. Our ship class, it will be a 'scenegraph sprite' 
class Ship : Node
{
public:
	//4. The max speed the ship can accelerate up to (pixels per ms)
	const float MAX_SPEED = 0.1;
	
	//5. This function is used by the physics system to tell if the ship collides with the asteroid
	void onCollideStart(Body a, Body b)
	{
		if(cast(Asteroid)b.getParent() !is null)
		{
			//6. If the ship does collide with the asteroid, we kill the ship
			kill();
		}
	}
	
	//7. Initialize our ship
	this()
	{
		// F. Circle is a physics class: radius is 13 and mass 100
		physics = new Circle(this, 13, 100);
		
		// G. Hook up our collision code to the physics circle
		physics.sigCollideStart.attach(&onCollideStart);

		// H. put ship at center of screen
		physics.translation = Point(arc.window.getWidth / 2, arc.window.getHeight / 2);
		
		reset();
		
		super(spacearea.area);
	}
	
	//8. Unregister sounds from the auto-process list 
	~this()
	{
		unregisterAutoProcessSound(laserSnd);
		unregisterAutoProcessSound(explodeSnd);
	}

	//9. Reset the ship, this is usually done after the ship has been killed
	void reset()
	{
		// start out with the static animation
		setGraphics(stillImage.makeNode());

		// put at center of screen
		physics.translation.set(arc.window.getWidth / 2, arc.window.getHeight / 2);
		physics.velocity.set(0,0);
		physics.angularVelocity = 0;

		// ship is alive
		alive = true;
	}

	//10. Accelerate by applying a forward force to the physics system 
	void accelerateForward()
	{
		physics.force += Point.fromPolar(0.01, physics.rotation - PI/2);
	}

	//11. Decelerate
	void accelerateBackward()
	{
		physics.force += Point.fromPolar(0.01, physics.rotation + PI/2);
	}
	
	//12. rotational acceleration to the left
	void rotateLeft()
	{
		physics.angularVelocity = -0.003;
	}

	//13. rotational acceleration to the right 
	void rotateRight()
	{
		physics.angularVelocity = 0.003;
	}
    
	//14. Process ship
	void process()
	{
		processControls();
	}
	
	//15. Process the ships controls
	void processControls()
	{
		// start with no angular velocity 
		physics.angularVelocity = 0;
		
		// keep the ship's speed from going above the max speed 
		if(physics.velocity.lengthSquared > MAX_SPEED*MAX_SPEED)
			physics.velocity = physics.velocity.normaliseCopy() * MAX_SPEED;
		
		// process the ships controls if the ship is alive 
		if(alive)
		{
			if (arc.input.keyDown(ARC_LEFT))
				rotateLeft();
	
			if (arc.input.keyDown(ARC_RIGHT))
				rotateRight();
			
			if (arc.input.keyPressed(ARC_UP) || arc.input.keyPressed(ARC_DOWN))
				setGraphics(moveAnimation.makeNode());
			
			if ((arc.input.keyReleased(ARC_UP) && arc.input.keyUp(ARC_DOWN)) ||
			    (arc.input.keyReleased(ARC_DOWN) && arc.input.keyUp(ARC_UP)))
				setGraphics(stillImage.makeNode());
	
			// speedup when moving, slow down when not
			if (arc.input.keyDown(ARC_UP))
				accelerateForward();
			if (arc.input.keyDown(ARC_DOWN))
				accelerateBackward(); 

			// fire laser when control is hit
			if (arc.input.keyPressed(ARC_LCTRL))
			{
				// play the laser sound
				laserSnd.play();
                
				// add a new laser bolt to the scenegraph with specified position, angle, and rotation
				LaserBolt.addLaser(
					physics.translation + Point.fromPolar(32, physics.rotation - PI/2),
					physics.rotation
					);
				
				scoreboard.scoreBoard.addScore(-10);
			}
		}
	}

	//16. Kill the ship and only if ship is alive 
	void kill()
	{
		if(alive)
		{
			// play explode sound effect
			explodeSnd.stop();
			explodeSnd.play();
			
			// start the exploding animation 
			setGraphics(new ShipExplosion(null));
			
			// ship is now dead
			alive = false;
		}
	}
	
	void setGraphics(Node to)
	{
		if(graphics)
			graphics.destroy();
		to.setParent(physics);
		graphics = to;
	}

private:
	//18. Our ships variables, animation frames, physics body, and alive boolean 
	Body physics;
	Node graphics;
	bool alive=true;

	static Image stillImage;
	static LinearAnimation explosionAnimation, moveAnimation;
	
	//17. Explode and Laser sound effects
	static Sound explodeSnd, laserSnd; 

	static void loadResources()
	{
		// A. The frame we define when the ship is not moving 
		stillImage = new Image(Texture("astbin/ship/still.png"));

		// B. When the ship is moving, this moveAnimation is shown (checkout move.png to see how animation is set up) 
		// the argument to linear animation are /animation/, /size of anim/, /numframes/, /time lapse between frames/, and /anim-loop style/ 
		moveAnimation = new LinearAnimation(new Image(Texture("astbin/ship/move.png"), Rect(32,32)), 3, 50,  LinearAnimation.Direction.Circular);

		// C. When the ship dies, this explode animation is shown
		explosionAnimation = new LinearAnimation(new Image(Texture("astbin/ship/explosion2.png"), Rect(32,32)), 7, 100);

		// D. sound effects 
		explodeSnd = new Sound(new SoundFile("astbin/explosion.wav"));
		laserSnd = new Sound(new SoundFile("astbin/ship/laser.wav"));
		
		// E. Instead of calling process manually on sound effects, we can add them to an auto-process list 
		// and arc.sound.process() will process these sounds automatically
		registerAutoProcessSound(explodeSnd);
		registerAutoProcessSound(laserSnd);        
	}
	mixin onWindowOpen!(loadResources);
}
