﻿// 1. Name of our module
module starfield;

// 2. Imports, need drawing, scenegraph, and math functions. The window will help us place stars on the screen
import
		arc.draw.all,
		arc.scenegraph.all,
		arc.math.all;

import arc.window;

Starfield stars;

// 3. Our starfield class will manage the background stars
class Starfield : Node, IDrawable
{
public:
	// 4. Initialize our stars with random colors and locations
	this()
	{
		super(rootNode);
	
		// pick a random amount of stars
		stars.length = randomRange(150,200);

		// and initialize them with random colors and locations
		foreach(inout star; stars)
			star.init();
	}

	// 5. Draw all the stars in the starfield
	override void draw()
	{
		foreach(inout star; stars)
			star.draw();
	}
	
private:
	// 6. Our array of stars
	Star[] stars; 
}

// 7. Definition of a single star
struct Star
{
	// 8. Variables required of our stars
	Point position;
	Color color; 
	uint size;
	bool glowUp = false; 
	int glowRate;
	uint glowRateCounter=0;     
    
	// 9. Initialize random colors and positions
	void init()
	{
		size = randomRange(1,3);
		
		position.x = randomRange(0, arc.window.getWidth);
		position.y = randomRange(0, arc.window.getHeight); 
			
		color.r = randomRange(0,255) / 255.; 
		color.g = randomRange(0,255) / 255.;
		color.b = randomRange(0,255) / 255.;
		color.a = randomRange(0,255) / 255.;
		
		glowRate = randomRange(1,5);
	}

	// 10. Draw the stars at given position and color
	void draw()
	{		
		if (glowRateCounter++ == glowRate)
		{
			if (glowUp)
				color.a += 0.05;
			else
				color.a -= 0.05; 
			
			if (color.a >= 1.)
				glowUp = false;
			
			if (color.a <= 0.)
				glowUp = true; 
			
			glowRateCounter = 0;
		}
        
		for(uint xi = 0; xi < size; ++xi)
			for(uint yi = 0; yi < size; ++yi)
				drawPixel(position + Point(xi, yi), color);
	}
}